import Token from '../token/token';
export declare type AnyNodeType = Node | Statement | Expression | ASTProgram;
export interface Node {
    TokenLiteral(): string;
    String(): string;
}
export declare class ASTProgram implements Node {
    Statements: Statement[];
    constructor(statements?: never[]);
    TokenLiteral(): string;
    String(): string;
}
export interface Statement extends Node {
}
export interface Expression extends Node {
}
export declare class LetStatement implements Statement {
    Token: Token;
    Name: Identifier;
    Value: Expression | Identifier;
    constructor(token: Token, name: Identifier, value: Expression | Identifier);
    TokenLiteral(): string;
    String(): string;
}
export declare class ReturnStatement implements Statement {
    Token: Token;
    ReturnValue: Expression | null;
    constructor(token: Token, returnValue?: null);
    TokenLiteral(): string;
    String(): string;
}
export declare class ExpressionStatement implements Statement {
    Token: Token;
    Expression: Expression;
    constructor(token: Token, expression: Expression);
    TokenLiteral(): string;
    String(): string;
}
export declare class Identifier implements Expression {
    Token: Token;
    Value: string;
    constructor(token: Token, value: string);
    TokenLiteral(): string;
    String(): string;
}
export declare class Comment implements Expression {
    Token: Token;
    Value: string;
    constructor(token: Token, value: string);
    TokenLiteral(): string;
    String(): string;
}
export declare class IntegerLiteral implements Expression {
    Token: Token;
    Value: number;
    constructor(token: Token, value: number);
    TokenLiteral(): string;
    String(): string;
}
export declare class FloatLiteral implements Expression {
    Token: Token;
    Value: number;
    constructor(token: Token, value: number);
    TokenLiteral(): string;
    String(): string;
}
export declare class PrefixExpression implements Expression {
    Token: Token;
    Operator: string;
    Right: Expression;
    constructor(token: Token, operator: string, right: Expression);
    TokenLiteral(): string;
    String(): string;
}
export declare class InfixExpression implements Expression {
    Token: Token;
    Left: Expression;
    Operator: string;
    Right: Expression | null;
    constructor(token: Token, left: Expression, operator: string, right: Expression | null);
    TokenLiteral(): string;
    String(): string;
}
export declare class AstBoolean implements Expression {
    Token: Token;
    Value: boolean;
    constructor(token: Token, value: boolean);
    TokenLiteral(): string;
    String(): string;
}
export declare class IfExpression implements Expression {
    Token: Token;
    Condition: Expression;
    Consequence: BlockStatement;
    Alternative: BlockStatement | null;
    constructor(token: Token, expression: Expression, consequence: BlockStatement, alternative?: BlockStatement | null);
    TokenLiteral(): string;
    String(): string;
}
export declare class BlockStatement implements Statement {
    Token: Token;
    Statements: Statement[];
    constructor(token: Token, statements?: never[]);
    TokenLiteral(): string;
    String(): string;
}
export declare class WhileLiteral implements Statement {
    Token: Token;
    Expression: Expression;
    Body: BlockStatement;
    constructor(token: Token, expression: Expression, body: BlockStatement);
    TokenLiteral(): string;
    String(): string;
}
export declare class FunctionLiteral implements Statement {
    Token: Token;
    Parameters: Identifier[];
    Body: BlockStatement;
    constructor(token: Token, parameters: Identifier[] | undefined, body: BlockStatement);
    TokenLiteral(): string;
    String(): string;
}
export declare class CallExpression implements Expression {
    Token: Token;
    Function: Expression;
    Arguments: Expression[];
    constructor(token: Token, func: Expression, args?: Expression[]);
    TokenLiteral(): string;
    String(): string;
}
export declare class StringLiteral implements Expression {
    Token: Token;
    Value: string;
    constructor(token: Token, value: string);
    TokenLiteral(): string;
    String(): string;
}
export declare class ArrayLiteral implements Expression {
    Token: Token;
    Elements: Expression[];
    constructor(token: Token, elements?: Expression[]);
    TokenLiteral(): string;
    String(): string;
}
export declare class IndexExpression implements Expression {
    Token: Token;
    Left: Expression;
    Index: Expression;
    constructor(token: Token, left: Expression, index: Expression);
    TokenLiteral(): string;
    String(): string;
}
export declare class HashLiteral implements Expression {
    Token: Token;
    Pairs: Map<Expression, Expression>;
    constructor(token: Token, pairs: Map<Expression, Expression>);
    TokenLiteral(): string;
    String(): string;
}
