"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const object_1 = require("../../object/object");
const evaluator_1 = require("../evaluator");
exports.default = new object_1.Builtin(function (env, ...args) {
    if (args.length !== 1) {
        return evaluator_1.newError('wrong number of arguments. got=%s, want=1', args.length);
    }
    if (args[0].Type() !== object_1.STRING_OBJ || !(args[0] instanceof object_1.OString)) {
        return evaluator_1.newError('argument to `json_parse` must be STRING, got %s', args[0].Type());
    }
    let obj = JSON.parse(args[0].Inspect());
    return json_parse(obj);
});
function json_parse(input) {
    if (Array.isArray(input)) {
        return new object_1.OArray(input.map(v => json_parse(v)));
    }
    else if (input === null) {
        return evaluator_1.NULL;
    }
    else if (typeof input === 'object') {
        let pairs = new Map();
        Object.keys(input).forEach((key) => {
            let hashKey = new object_1.OString(key);
            let hashed = hashKey.HashKey();
            pairs.set(hashed.Match, new object_1.HashPair(hashKey, json_parse(input[key])));
        });
        return new object_1.OHash(pairs);
    }
    else if (typeof input === 'number') {
        return new object_1.OInteger(input);
    }
    else if (typeof input === 'string') {
        return new object_1.OString(input);
    }
    return evaluator_1.NULL;
}
//# sourceMappingURL=json_parse.js.map