"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const token_1 = require("../token/token");
const lexer_1 = __importDefault(require("./lexer"));
function TestLexer(t) {
    console.log('║  └ TestNextToken');
    TestNextToken(t);
}
exports.TestLexer = TestLexer;
function TestNextToken(t) {
    let input = `let five = 5;
let ten = 10;

let add = fn(x, y) {
  x + y;
};

let result = add(five, ten);
!-/*5;
5 < 10 > 5;

if (5 < 10) {
  return true;
} else {
  return false;
}

10 == 10;
10 != 9;
"foobar"
"foo bar"
[1, 2];
{"foo": "bar"}
1 >= 2
2 <= 1
123.45
`;
    let tests = [
        [token_1.TokenType.LET, 'let'],
        [token_1.TokenType.IDENT, 'five'],
        [token_1.TokenType.ASSIGN, '='],
        [token_1.TokenType.INT, '5'],
        [token_1.TokenType.SEMICOLON, ';'],
        [token_1.TokenType.LET, 'let'],
        [token_1.TokenType.IDENT, 'ten'],
        [token_1.TokenType.ASSIGN, '='],
        [token_1.TokenType.INT, '10'],
        [token_1.TokenType.SEMICOLON, ';'],
        [token_1.TokenType.LET, 'let'],
        [token_1.TokenType.IDENT, 'add'],
        [token_1.TokenType.ASSIGN, '='],
        [token_1.TokenType.FUNCTION, 'fn'],
        [token_1.TokenType.LPAREN, '('],
        [token_1.TokenType.IDENT, 'x'],
        [token_1.TokenType.COMMA, ','],
        [token_1.TokenType.IDENT, 'y'],
        [token_1.TokenType.RPAREN, ')'],
        [token_1.TokenType.LBRACE, '{'],
        [token_1.TokenType.IDENT, 'x'],
        [token_1.TokenType.PLUS, '+'],
        [token_1.TokenType.IDENT, 'y'],
        [token_1.TokenType.SEMICOLON, ';'],
        [token_1.TokenType.RBRACE, '}'],
        [token_1.TokenType.SEMICOLON, ';'],
        [token_1.TokenType.LET, 'let'],
        [token_1.TokenType.IDENT, 'result'],
        [token_1.TokenType.ASSIGN, '='],
        [token_1.TokenType.IDENT, 'add'],
        [token_1.TokenType.LPAREN, '('],
        [token_1.TokenType.IDENT, 'five'],
        [token_1.TokenType.COMMA, ','],
        [token_1.TokenType.IDENT, 'ten'],
        [token_1.TokenType.RPAREN, ')'],
        [token_1.TokenType.SEMICOLON, ';'],
        [token_1.TokenType.BANG, '!'],
        [token_1.TokenType.MINUS, '-'],
        [token_1.TokenType.SLASH, '/'],
        [token_1.TokenType.ASTERISK, '*'],
        [token_1.TokenType.INT, '5'],
        [token_1.TokenType.SEMICOLON, ';'],
        [token_1.TokenType.INT, '5'],
        [token_1.TokenType.LT, '<'],
        [token_1.TokenType.INT, '10'],
        [token_1.TokenType.GT, '>'],
        [token_1.TokenType.INT, '5'],
        [token_1.TokenType.SEMICOLON, ';'],
        [token_1.TokenType.IF, 'if'],
        [token_1.TokenType.LPAREN, '('],
        [token_1.TokenType.INT, '5'],
        [token_1.TokenType.LT, '<'],
        [token_1.TokenType.INT, '10'],
        [token_1.TokenType.RPAREN, ')'],
        [token_1.TokenType.LBRACE, '{'],
        [token_1.TokenType.RETURN, 'return'],
        [token_1.TokenType.TRUE, 'true'],
        [token_1.TokenType.SEMICOLON, ';'],
        [token_1.TokenType.RBRACE, '}'],
        [token_1.TokenType.ELSE, 'else'],
        [token_1.TokenType.LBRACE, '{'],
        [token_1.TokenType.RETURN, 'return'],
        [token_1.TokenType.FALSE, 'false'],
        [token_1.TokenType.SEMICOLON, ';'],
        [token_1.TokenType.RBRACE, '}'],
        [token_1.TokenType.INT, '10'],
        [token_1.TokenType.EQ, '=='],
        [token_1.TokenType.INT, '10'],
        [token_1.TokenType.SEMICOLON, ';'],
        [token_1.TokenType.INT, '10'],
        [token_1.TokenType.NOT_EQ, '!='],
        [token_1.TokenType.INT, '9'],
        [token_1.TokenType.SEMICOLON, ';'],
        [token_1.TokenType.STRING, 'foobar'],
        [token_1.TokenType.STRING, 'foo bar'],
        [token_1.TokenType.LBRACKET, '['],
        [token_1.TokenType.INT, '1'],
        [token_1.TokenType.COMMA, ','],
        [token_1.TokenType.INT, '2'],
        [token_1.TokenType.RBRACKET, ']'],
        [token_1.TokenType.SEMICOLON, ';'],
        [token_1.TokenType.LBRACE, '{'],
        [token_1.TokenType.STRING, 'foo'],
        [token_1.TokenType.COLON, ':'],
        [token_1.TokenType.STRING, 'bar'],
        [token_1.TokenType.RBRACE, '}'],
        [token_1.TokenType.INT, '1'],
        [token_1.TokenType.GTE, '>='],
        [token_1.TokenType.INT, '2'],
        [token_1.TokenType.INT, '2'],
        [token_1.TokenType.LTE, '<='],
        [token_1.TokenType.INT, '1'],
        [token_1.TokenType.FLOAT, '123.45'],
        [token_1.TokenType.EOF, ''],
    ];
    let l = new lexer_1.default(input);
    for (let i in tests) {
        let tt = tests[i];
        let tok = l.NextToken();
        t.Assert(tok.Type === tt[0], `tests[${i}] - type wrong. expected=${tt[0]}, got=${tok.Type}`);
        t.Assert(tok.Literal === tt[1], `tests[${i}] - literal wrong. expected=${tt[1]}, got=${tok.Literal}`);
    }
}
exports.TestNextToken = TestNextToken;
//# sourceMappingURL=lexer-test.js.map