"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const token_1 = __importStar(require("../token/token"));
const position_1 = __importDefault(require("../token/position"));
class Lexer {
    constructor(input) {
        this.line = 1;
        this.column = 0;
        this.position = 0;
        this.readPosition = 0;
        this.ch = null;
        if (typeof input !== 'string') {
            throw new Error('Lexer: only `string` input allowed');
        }
        this.input = input;
        this.readChar();
    }
    NextToken() {
        let tok;
        this.skipWhitespace();
        let pos = new position_1.default(this.position, this.line, this.column);
        switch (this.ch) {
            case '=':
                if (this.peekChar() === '=') {
                    let ch = this.ch;
                    this.readChar();
                    let literal = ch + this.ch;
                    tok = new token_1.default(token_1.TokenType.EQ, literal, pos);
                }
                else {
                    tok = new token_1.default(token_1.TokenType.ASSIGN, this.ch, pos);
                }
                break;
            case '+':
                tok = new token_1.default(token_1.TokenType.PLUS, this.ch, pos);
                break;
            case '-':
                tok = new token_1.default(token_1.TokenType.MINUS, this.ch, pos);
                break;
            case '!':
                if (this.peekChar() === '=') {
                    let ch = this.ch;
                    this.readChar();
                    let literal = ch + this.ch;
                    tok = new token_1.default(token_1.TokenType.NOT_EQ, literal, pos);
                }
                else {
                    tok = new token_1.default(token_1.TokenType.BANG, this.ch, pos);
                }
                break;
            case '/':
                if (this.peekChar() === '/') {
                    this.readChar();
                    tok = new token_1.default(token_1.TokenType.COMMENT, this.readComment(), pos);
                    break;
                }
                else {
                    tok = new token_1.default(token_1.TokenType.SLASH, this.ch, pos);
                }
                break;
            case '#':
                tok = new token_1.default(token_1.TokenType.COMMENT, this.readComment(), pos);
                break;
            case '*':
                if (this.peekChar() === '*') {
                    let ch = this.ch;
                    this.readChar();
                    let literal = ch + this.ch;
                    tok = new token_1.default(token_1.TokenType.EXPONENT, literal, pos);
                }
                else {
                    tok = new token_1.default(token_1.TokenType.ASTERISK, this.ch, pos);
                }
                break;
            case '%':
                tok = new token_1.default(token_1.TokenType.REM, this.ch, pos);
                break;
            case '<':
                if (this.peekChar() === '=') {
                    let ch = this.ch;
                    this.readChar();
                    let literal = ch + this.ch;
                    tok = new token_1.default(token_1.TokenType.LTE, literal, pos);
                }
                else {
                    tok = new token_1.default(token_1.TokenType.LT, this.ch, pos);
                }
                break;
            case '>':
                if (this.peekChar() === '=') {
                    let ch = this.ch;
                    this.readChar();
                    let literal = ch + this.ch;
                    tok = new token_1.default(token_1.TokenType.GTE, literal, pos);
                }
                else {
                    tok = new token_1.default(token_1.TokenType.GT, this.ch, pos);
                }
                break;
            case ';':
                tok = new token_1.default(token_1.TokenType.SEMICOLON, this.ch, pos);
                break;
            case ',':
                tok = new token_1.default(token_1.TokenType.COMMA, this.ch, pos);
                break;
            case '{':
                tok = new token_1.default(token_1.TokenType.LBRACE, this.ch, pos);
                break;
            case '}':
                tok = new token_1.default(token_1.TokenType.RBRACE, this.ch, pos);
                break;
            case '(':
                tok = new token_1.default(token_1.TokenType.LPAREN, this.ch, pos);
                break;
            case ')':
                tok = new token_1.default(token_1.TokenType.RPAREN, this.ch, pos);
                break;
            case '[':
                tok = new token_1.default(token_1.TokenType.LBRACKET, this.ch, pos);
                break;
            case ']':
                tok = new token_1.default(token_1.TokenType.RBRACKET, this.ch, pos);
                break;
            case '"':
                tok = new token_1.default(token_1.TokenType.STRING, this.readString(), pos);
                break;
            case "'":
                tok = new token_1.default(token_1.TokenType.STRING, this.readString("'"), pos);
                break;
            case ':':
                tok = new token_1.default(token_1.TokenType.COLON, this.ch, pos);
                break;
            case 0:
                tok = new token_1.default(token_1.TokenType.EOF, '', pos);
                break;
            default:
                if (isLetter(this.ch)) {
                    let literal = this.readIdentifier();
                    return new token_1.default(token_1.LookupIdent(literal), literal, pos);
                }
                else if (isDigit(this.ch)) {
                    let literal = this.readNumber();
                    if (isFloat(literal)) {
                        return new token_1.default(token_1.TokenType.FLOAT, literal, pos);
                    }
                    return new token_1.default(token_1.TokenType.INT, literal, pos);
                }
                else {
                    tok = new token_1.default(token_1.TokenType.ILLEGAL, '' + this.ch, pos);
                }
        }
        this.readChar();
        return tok;
    }
    skipWhitespace() {
        while (this.ch === ' ' || this.ch === '\t' || this.ch === '\n' || this.ch === '\r') {
            this.readChar();
        }
    }
    readChar() {
        if (this.readPosition >= this.input.length) {
            this.ch = 0;
        }
        else {
            this.ch = this.input[this.readPosition];
        }
        if (this.ch === '\n') {
            this.column = 0;
            this.line++;
        }
        this.position = this.readPosition;
        this.column++;
        this.readPosition++;
    }
    readString(type = '"') {
        let position = this.position + 1;
        while (true) {
            this.readChar();
            if (this.ch === type || this.ch === 0) {
                break;
            }
        }
        return this.input.slice(position, this.position);
    }
    readComment() {
        let position = this.position + 1;
        while (true) {
            this.readChar();
            if (this.ch === '\n' || this.ch === '\r' || this.ch === 0) {
                break;
            }
        }
        return this.input.slice(position, this.position).trim();
    }
    peekChar() {
        if (this.readPosition >= this.input.length) {
            return 0;
        }
        else {
            return this.input[this.readPosition];
        }
    }
    readIdentifier() {
        let position = this.position;
        while (isLetter(this.ch)) {
            this.readChar();
        }
        return this.input.slice(position, this.position);
    }
    readNumber() {
        let position = this.position;
        while (isDigit(this.ch)) {
            this.readChar();
        }
        return this.input.slice(position, this.position);
    }
}
exports.default = Lexer;
function isLetter(ch) {
    if (!ch)
        return false;
    return ('a' <= ch && ch <= 'z') || ('A' <= ch && ch <= 'Z') || ch === '_' || ch === '-';
}
function isDigit(ch) {
    if (!ch)
        return false;
    if (typeof ch === 'number')
        return true;
    return '0123456789.'.indexOf(ch) !== -1;
}
function isFloat(n) {
    let val = parseFloat(n);
    return !isNaN(val) && n.indexOf('.') !== -1;
}
//# sourceMappingURL=lexer.js.map