import { Identifier, BlockStatement } from '../ast/ast';
import Environment from './environment';
export declare const BOOLEAN_OBJ = "BOOLEAN", BUILTIN_OBJ = "BUILTIN", ERROR_OBJ = "ERROR", COMMENT_OBJ = "COMMENT", FUNCTION_OBJ = "FUNCTION", INTEGER_OBJ = "INTEGER", FLOAT_OBJ = "FLOAT", NULL_OBJ = "NULL", RETURN_VALUE_OBJ = "RETURN_VALUE", STRING_OBJ = "STRING", ARRAY_OBJ = "ARRAY", HASH_OBJ = "HASH";
export declare type AnyObject = OObject | OInteger | OFloat | OBoolean | OString | ONull;
export declare type NullableOObject = OObject | null;
export declare type NullableString = string | null;
export declare type Hashable = OBoolean | OInteger | OFloat | OString;
export declare type HasValue = OBoolean | OInteger | OFloat | ReturnValue | OString | OArray | OHash;
export default interface OObject {
    Type(): string;
    Inspect(): string;
    toString(): string;
    toValue(): any;
}
export declare class HashKey {
    Type: string;
    Value: number;
    Match: string;
    constructor(type: string, value: number);
}
export interface BuiltinFunction {
    (env: Environment, ...args: OObject[]): OObject;
}
export declare class OBoolean implements OObject {
    Value: boolean;
    constructor(value: boolean);
    Type(): string;
    Inspect(): "true" | "false";
    toString(): "true" | "false";
    toValue(): boolean;
    HashKey(): HashKey;
}
export declare class OInteger implements OObject {
    Value: number;
    constructor(value: number);
    Type(): string;
    Inspect(): string;
    toString(): string;
    toValue(): number;
    HashKey(): HashKey;
}
export declare class OFloat implements OObject {
    Value: number;
    constructor(value: number);
    Type(): string;
    Inspect(): string;
    toString(): string;
    toValue(): number;
    HashKey(): HashKey;
}
export declare class ONull implements OObject {
    Type(): string;
    Inspect(): "" | "null";
    toString(): string;
    toValue(): null;
}
export declare class ReturnValue implements OObject {
    Value: OObject;
    constructor(value: OObject);
    Type(): string;
    Inspect(): string;
    toString(): string;
    toValue(): any;
}
export declare class OError implements OObject {
    Message: string;
    constructor(message: string);
    Type(): string;
    Inspect(): string;
    toString(): string;
    toValue(): Error;
}
export declare class OFunction implements OObject {
    Parameters: Identifier[];
    Body: BlockStatement;
    Env: Environment;
    constructor(parameters: Identifier[], body: BlockStatement, env: Environment);
    Type(): string;
    Inspect(): string;
    toString(): string;
    toValue(): Error;
}
export declare class OString implements OObject {
    Value: string;
    constructor(value: string);
    Type(): string;
    Inspect(): string;
    toString(): string;
    toValue(): string;
    HashKey(): HashKey;
}
export declare class OComment implements OObject {
    Message: string;
    constructor(message: string);
    Type(): string;
    Inspect(): string;
    toString(): string;
    toValue(): string;
    HashKey(): HashKey;
}
export declare class Builtin implements OObject {
    Fn: BuiltinFunction;
    constructor(fn: BuiltinFunction);
    Type(): string;
    Inspect(): string;
    toString(): string;
    toValue(): Error;
}
export declare class OArray implements OObject {
    Elements: OObject[];
    constructor(elements: OObject[]);
    Type(): string;
    Inspect(): string;
    toString(): string;
    toValue(): NullableString[];
}
export declare type HashPairs = Map<string, HashPair>;
export declare class HashPair {
    Key: OObject;
    Value: OObject;
    constructor(key: OObject, value: OObject);
}
export declare class OHash implements OObject {
    Pairs: HashPairs;
    constructor(pairs: HashPairs);
    Type(): string;
    Inspect(): string;
    toString(): string;
    toValue(): {
        [index: string]: any;
    };
}
