"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const process_1 = __importDefault(require("process"));
const lexer_1 = __importDefault(require("../lexer/lexer"));
const parser_1 = __importDefault(require("../parser/parser"));
const evaluator_1 = __importDefault(require("../evaluator/evaluator"));
const environment_1 = require("../object/environment");
const object_1 = require("../object/object");
const PROMPT = '>> ';
function Start(input, output, quiet = false) {
    if (!quiet)
        output.write(PROMPT);
    let env = environment_1.NewEnvironment();
    env.Logger.Follow((date, messages) => {
        console.log(date, ...messages);
    });
    let lastOutput;
    input.on('data', (data) => {
        if (data === 'exit\n')
            process_1.default.exit(0);
        let l = new lexer_1.default(data);
        let p = new parser_1.default(l);
        let program = p.ParseProgram();
        if (p.Errors().length !== 0) {
            printParserErrors(process_1.default.stderr, p.Errors());
            if (!quiet)
                output.write(PROMPT);
            return;
        }
        let evaluated = evaluator_1.default(program, env);
        if (evaluated !== null) {
            lastOutput = evaluated;
            if (!quiet)
                output.write(evaluated.Inspect());
            if (!quiet)
                output.write('\n');
        }
        if (!quiet)
            output.write(PROMPT);
    });
    input.on('end', () => {
        if (lastOutput instanceof object_1.OInteger)
            process_1.default.exit(lastOutput.Value);
        process_1.default.exit(0);
    });
    input.on('close', () => {
        process_1.default.exit(0);
    });
    input.on('error', (err) => {
        process_1.default.stderr.write(`\x1b[31m==> \t${err}\n\x1b[0m`);
    });
}
exports.default = Start;
function printParserErrors(output, errors) {
    output.write(`\x1b[31m`);
    output.write('Woops! We ran into some monkey business here!\n');
    output.write('    Parser errors:\n');
    errors.forEach((msg) => output.write(`\t${msg}\n`));
    output.write('\x1b[0m');
}
//# sourceMappingURL=repl.js.map