"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ast_test_1 = require("./ast/ast-test");
const evaluator_test_1 = require("./evaluator/evaluator-test");
const lexer_test_1 = require("./lexer/lexer-test");
const parser_test_1 = require("./parser/parser-test");
const object_test_1 = require("./object/object-test");
class Test {
    constructor() {
        this.TotalCount = 0;
        this.PassCount = 0;
        this.FailCount = 0;
        this.SkipCount = 0;
    }
    Error(...args) {
        this.TotalCount++;
        this.FailCount++;
        console.error('\x1b[31m%s\x1b[0m', ...args);
    }
    Fail() {
        this.TotalCount++;
        this.FailCount++;
    }
    Failed() {
        return !!this.FailCount;
    }
    Fatal(...args) {
        this.TotalCount++;
        this.FailCount++;
        console.error('\x1b[31m%s\x1b[0m', ...args);
        throw new Error('FAIL');
    }
    Log(...args) {
        console.log(...args);
    }
    Logf(format, ...args) {
        console.log(format, ...args);
    }
    Skip(...args) {
        this.SkipCount++;
        console.info('\x1b[34mSkipped %s\x1b[0m', ...args);
    }
    Skipf(format, ...args) {
        this.SkipCount++;
        console.info(`\x1b[34mSkipped ${format}\x1b[0m`, ...args);
    }
    Skipped() {
        return !!this.SkipCount;
    }
    Assert(test, message, ...args) {
        this.TotalCount++;
        if (test) {
            this.PassCount++;
            return;
        }
        this.Errorf(message, ...args);
    }
    Fatalf(format, ...args) {
        this.TotalCount++;
        this.FailCount++;
        throw new Error(format);
    }
    Errorf(format, ...args) {
        this.TotalCount++;
        this.FailCount++;
        var err = new Error();
        debugger;
        console.error(`\x1b[31m${format}\x1b[0m`, ...args);
    }
    FailNow() {
        this.TotalCount++;
        this.FailCount++;
        throw new Error('FAIL');
    }
}
exports.default = Test;
function main() {
    let t = new Test();
    console.log(`\x1b[36m╔═════════════>\n║ \x1b[1mStart tests\n\x1b[0m\x1b[36m║\x1b[0m`);
    console.time('║ took');
    try {
        console.log('║\n╠══╕ \x1b[1mStart TestAst\x1b[0m');
        ast_test_1.TestAst(t);
        console.log('║\n╠══╕ \x1b[1mStart TestLexer\x1b[0m');
        lexer_test_1.TestLexer(t);
        console.log('║\n╠══╕ \x1b[1mStart TestParser\x1b[0m');
        parser_test_1.TestParser(t);
        console.log('║\n╠══╕ \x1b[1mStart TestEval\x1b[0m');
        evaluator_test_1.TestEval(t);
        console.log('║\n╠══╕ \x1b[1mStart TestObject\x1b[0m');
        object_test_1.TestObject(t);
    }
    catch { }
    console.log('║');
    console.timeEnd('║ took');
    if (t.FailCount > 0) {
        console.error(`\x1b[31m║\n╠══> \x1b[1m${t.FailCount} of ${t.TotalCount}\x1b[0m\x1b[31m tests failed ❌\n║\x1b[0m`);
        console.log(`\x1b[36m║ \x1b[1mEnd tests\n\x1b[0m\x1b[36m╚═══════════>\x1b[0m`);
        process.exit(1);
        return;
    }
    if (t.SkipCount > 0) {
        console.info(`\x1b[34m║\n╠══> \x1b[1m${t.SkipCount}\x1b[0m\x1b[34m tests skipped ℹ️\n║\x1b[0m`);
    }
    console.info(`\x1b[32m║\n╠══> \x1b[1m${t.PassCount} of ${t.TotalCount}\x1b[0m\x1b[32m assertions passed ✅\n║\x1b[0m`);
    console.log(`\x1b[36m║ \x1b[1mEnd tests\n\x1b[0m\x1b[36m╚═══════════>\x1b[0m`);
    process.exit(0);
}
exports.main = main;
main();
//# sourceMappingURL=test.js.map